package net.pricefx.common.apibuilder;

import net.pricefx.common.api.chart.FilterOperator;
import net.pricefx.common.apibuilder.chart.Filters;

@SuppressWarnings("rawtypes")
public abstract class AbstractSeries<ATTACHEDTO extends ObjectIdentity, T extends AbstractSeries> extends BacklinkBuilder<ATTACHEDTO> {
	
	public AbstractSeries(String instanceId, String classId, ATTACHEDTO parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, parentChartObjectIdentity);
	}

	private Filters<T> filters;
	private Filters<T> productsFilter;
	private Filters<T> customersFilter;
	
	@SuppressWarnings("unchecked")
	public final T setLabel(String label) {
		addInvocation("setLabel",label);
		return (T) this;
	}
			
	@SuppressWarnings("unchecked")
	public final T setDatamart(String datamart) {
		addInvocation("setDatamart",datamart);
		return (T) this;
	}
	
	@SuppressWarnings("unchecked")
	public final T setCurrency(String currency) {
		addInvocation("setCurrency",currency);
		return (T) this;
	}
	
	public final Filters<T> getFilters() { // backward compatibility
		return addFilters();
	}
	
	public final Filters<T> addFilters() {
		return addFilters(FilterOperator.AND);
	}
	
	@SuppressWarnings({ "unchecked" })
	public final Filters<T> addFilters(FilterOperator operator) {
		String id = addInvocation("addFilters",operator);
		if (filters == null) 
			filters = new Filters(this, id, "Filters");
		return filters;
	}
	
	@SuppressWarnings({ "unchecked" })
	public final Filters<T> addProductsFilter() {
		String id = addInvocation("addProductsFilter");
		if (productsFilter == null) 
			productsFilter = new Filters(this, id, "ProductsFilter");
		return productsFilter;
	}
	
	@SuppressWarnings({ "unchecked" })
	public final Filters<T> addCustomersFilter() {
		String id = addInvocation("addCustomersFilter");
		if (customersFilter == null) 
			customersFilter = new Filters(this, id, "CustomersFilter");
		return customersFilter;
	}
	
	@SuppressWarnings("unchecked")
	public final T addDimFilter(String dimension, String value) {
		addInvocation("addDimFilter", dimension, value);
		return (T) this;
	}
}