package net.pricefx.common.apibuilder;


public abstract class ObjectIdentity {

	public static final ObjectIdentity NONE = new ObjectIdentity() {};
	protected final IdGenerator idGenerator;
	protected Methods m = new Methods();
	protected final String instanceId;
	protected final String classId;
	protected final ObjectIdentity parentObjectIdentity;
	protected final Invocations<MethodInvocation> invocations;		
	
	
	private ObjectIdentity() {
		this.instanceId = "<NONE>";
		this.classId = "<NONE>";
		this.parentObjectIdentity = null;
		this.invocations = new Invocations<MethodInvocation>();
		this.idGenerator = new IdGenerator();
	}
	
	public ObjectIdentity(String instanceId, String classId, ObjectIdentity parentObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		if (instanceId == null) throw new IllegalArgumentException("instanceId cannot be null");
		if (classId == null) throw new IllegalArgumentException("classId cannot be null");
		if (parentObjectIdentity == null) throw new IllegalArgumentException("parentObjectIdentity cannot be null");
		if (invocations == null) throw new IllegalArgumentException("invocations cannot be null");
		
		this.instanceId = instanceId;
		this.classId = classId;
		this.parentObjectIdentity = parentObjectIdentity;
		this.invocations = invocations;
		this.idGenerator = idGenerator;
	}

	protected String addInvocation(String methodName,Object...arguments) {
		return addInvocation(methodName,this,arguments);
	}
	
	protected String addInvocation(String methodName,ObjectIdentity objectNameTokenIdentity,Object...arguments) {
		
		String methodNameToken = m.createMethodNameToken(objectNameTokenIdentity, methodName);
		String id = idGenerator.getNext();
		String methodId = m.createMethodId(this, id);
		MethodInvocation op = null;
		if(arguments != null && arguments.length > 0) {
			op = MethodInvocation.of(methodNameToken, methodId, arguments);
		} else {
			op = MethodInvocation.of(methodNameToken, methodId);
		}
		invocations.add(op);	
		return id;
	}
	
}