package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;

/**
 * This action starts exporting or archiving. It's a programmatic version
 * of the same action available in UI. As such it allows user to schedule the
 * action in calculation flow formula.
 */
public class ExportAction<ATTACHEDTO extends Help> extends Action {

	private String jobName;
	private String templateName;
	private Boolean doExport;
	private Boolean doArchive;
	private ATTACHEDTO backLink;
	
	public ExportAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations, String jobName) {
		super(invocations);
		this.backLink = backLink;
		this.jobName = jobName;
		this.doExport = false;
		this.doArchive = false;
	}

	/**
	 * Notifies system that export for selected template should be started. 
	 * 
	 * @param templateName template to use
	 */
	public ExportAction<ATTACHEDTO> doExport(String templateName) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".doExport", templateName));
		this.doExport = true;
		this.doArchive = false;
		this.templateName = templateName;
		return this;
	}
	
	/**
	 * Notifies system that archiving for selected template should be started.
	 * 
	 * @param templateName
	 *            template to use
	 */
	public ExportAction<ATTACHEDTO> doArchive(String templateName) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".doArchive", templateName));
		this.doArchive = true;
		this.doExport = false;
		this.templateName = templateName;
		return this;
	}
	
	public ATTACHEDTO back() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
		return backLink;
	}

	@Override
	public String getObjectName() {
		return backLink.getObjectName() + ".ExportAction";
	}

	public String getJobName() { return jobName; }
	public String getTemplateName() { return templateName; }
	public Boolean isExport() {	return doExport; }
	public Boolean isArchive() {	return doArchive; }
	
	@Override public String toString() { 
		return "ExportAction [jobName=" + jobName + ", templateName=" + templateName + ", export=" + doExport + ", archive=" + doArchive + ", backLink=" + backLink +  ", hashCode()=" + hashCode() + "]";
	}
	
}