package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.LimitPosition;
import net.pricefx.common.api.chart.LimitType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class BubbleChart extends AbstractBuilder {

	public BubbleChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private BubbleOptions options;
	private Dictionary<BubbleChart> dictionary;
	
	public BubbleSeries addSeries() {
		String id = addInvocation("addSeries");
		return new BubbleSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public BubbleOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new BubbleOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<BubbleChart> addAuxLine() {
		String id = addInvocation("addAuxLine");
		return new AuxLine<BubbleChart>(this, id, "AuxLine");
	}
	
	public Dictionary<BubbleChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.BUBBLE;
		return r;
	}

	
	public static class BubbleOptions extends AbstractOptions<BubbleChart,BubbleOptions> {

		public BubbleOptions(BubbleChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		public BubbleOptions setMarker(BigDecimal xValue, BigDecimal yValue) {
			addInvocation("setMarker", xValue, yValue);
			return this;
		}
		
		public BubbleOptions setXMin(BigDecimal xMin) {

			addInvocation("setXMin", xMin);
			return this;
		}
		
		public BubbleOptions setXMax(BigDecimal xMax) {

			addInvocation("setXMax", xMax);
			return this;
		}		
	}
	
	public static class BubbleSeries extends AbstractSeries<BubbleChart,BubbleSeries> {

		public BubbleSeries(BubbleChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public AggrFieldAxis<BubbleSeries> setAxisX(String axisX) {

			String id = addInvocation("setAxisX", axisX);
			return new AggrFieldAxis<BubbleSeries>(this, id, "AxisX");
		}
		
		public AggrFieldAxis<BubbleSeries> setAxisY(String axisY) {
			
			String id = addInvocation("setAxisY", axisY);
			return new AggrFieldAxis<BubbleSeries>(this, id, "AxisY");
		}

		public LimitToMeasure<BubbleChart.BubbleSeries> setLimitTo(LimitPosition limitPosition, BigDecimal amount, LimitType limitType) {
			String id = addInvocation("setLimitTo", limitPosition, amount, limitType);
			return new LimitToMeasure<>(this, id, "LimitToMeasure");
		}

		public BubbleSeries setAggregationBy(String aggregationBy) {
			
			addInvocation("setAggregationBy", aggregationBy);
			return this;
		}
		
		public BubbleSeries setBandBy(String bandBy) {
			
			addInvocation("setBandBy", bandBy);
			return this;
		}
		
		public BubbleSeries setShowBandByLegend(Boolean showBandByLegend) {
			
			addInvocation("setShowBandByLegend", showBandByLegend);
			return this;
		}
		
		public BubbleSize<BubbleSeries> setBubbleSize(String bubbleSize) {
			
			String id = addInvocation("setBubbleSize",bubbleSize);
			return new BubbleSize<BubbleSeries>(this, id, "BubbleSize", idGenerator, invocations);
		}
		
		public BubbleAdditionalMeasure<BubbleSeries> addAdditionalMeasure() {
			
			String id = addInvocation("addAdditionalMeasure");
			return new BubbleAdditionalMeasure<BubbleSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public TrendLine<BubbleSeries> withTrendLine() {
			
			String id = addInvocation("withTrendLine");
			return new TrendLine<BubbleSeries>(this, id, "TrendLine");
		}
		
		public Formula<BubbleSeries> setFormula() {

			String id = addInvocation("setFormula");
			return new Formula<BubbleSeries>(this, id, "Formula");
		}
		
		public BubbleSeries setGeneratedQueryDto(String generatedQueryDto) {
			
			addInvocation("setGeneratedQueryDto", generatedQueryDto);
			return this;
		}
		


	}
	
	public static class BubbleSize<ATTACHEDTO extends ObjectIdentity> extends AggrFieldAxis<ATTACHEDTO> {

		public BubbleSize(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(attachedTo, instanceId, classId);
		}
		
	}
	
	public static class BubbleAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,BubbleAdditionalMeasure<ATTACHEDTO>> {
 
		public BubbleAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}

		
		public Expression<BubbleAdditionalMeasure<ATTACHEDTO>> withExpression() { 
			String id = addInvocation("withExpression");
			return new Expression<BubbleAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}

	}
	
}