package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;

/**
 * Builder class for input type Customer {@link InputType#CUSTOMER}
 */
public class CustomerInputBuilder extends AbstractProductAndCustomerInputBuilder<CustomerInputBuilder, String> {

    private Boolean defaultFromParent;

    public CustomerInputBuilder(InputBuilderContext builderContext, String paramName) {
        super(builderContext, InputType.CUSTOMER, paramName);
    }

    /**
     * Set default from the parent flag of the CustomerInputBuilder.
     * It determines whether to look up the attribute value from the parent customer (if any)
     * <p>Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *         .createCustomerEntry()
     *         .setDefaultFromParent(true)
     *         .buildContextParameter()
     * </pre></blockquote>
     *
     * @param defaultFromParent true to set the default from the parent
     * @return self
     */
    public CustomerInputBuilder setDefaultFromParent(Boolean defaultFromParent) {
        this.defaultFromParent = defaultFromParent;
        return this;
    }

}
