package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.common.api.pa.FieldType;

import java.util.Collection;
import java.util.Set;

public class DMFieldInputBuilder extends AbstractInputBuilder<DMFieldInputBuilder, String> {

    private static final Set<InputType> VALID_INPUT_TYPES = Set.of(InputType.DMFIELD, InputType.DMFIELDS);

    public enum FieldKind {
        DIMENSION, NUMERIC, KEY, TIME;
    }

    public DMFieldInputBuilder(InputBuilderContext builderContext, InputType type, String paramName, String source) {
        super(builderContext, type, paramName);
        if (!VALID_INPUT_TYPES.contains(type)) {
            throw new IllegalArgumentException(String.format("Input type must be one of %s, got %s", VALID_INPUT_TYPES, type));
        }
        cp.addParameterConfigEntry(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_DM_SOURCE_NAME, source);
    }

    /**
     * @param kind Allowed {@link FieldKind}
     * @return self
     * @see #setFieldKinds(Collection)
     */
    public DMFieldInputBuilder addFieldKind(FieldKind kind) {
        return null;
    }

    /**
     * @param kinds Allowed {@link FieldKind}s
     * @return self
     * @see #setFieldKinds(Collection)
     */
    public DMFieldInputBuilder setFieldKinds(FieldKind... kinds) {
        return null;
    }

    /**
     * Set the kind of fields to be displayed in the drop down list.
     * <p>Example:
     * <blockquote>
     * <pre>
     * def builder = api.inputBuilderFactory().createDMFields("MyInput","DM.datamart_transaction")
     *                   .setFieldKinds(FieldKind.DIMENSION, FieldKind.KEY)  // to select fields that are dimension or key
     * return libs.InputLib.Input.inputParamBuilder(out, builder)
     * </pre>
     * </blockquote>
     *
     * @param kinds Allowed {@link FieldKind}s
     * @return self
     * @see FieldKind
     */
    public DMFieldInputBuilder setFieldKinds(Collection<FieldKind> kinds) {
        writeParameterCollection(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_DM_FIELD_KINDS, kinds);
        return this;
    }

    /**
     * @param type Allowed {@link FieldType}
     * @return self
     * @see #setFieldTypes(Collection)
     */
    public DMFieldInputBuilder addFieldType(FieldType type) {
        return null;
    }

    /**
     * @param types Allowed {@link FieldType}s
     * @return self
     * @see #setFieldTypes(Collection)
     */
    public DMFieldInputBuilder setFieldTypes(FieldType... types) {
        return null;
    }

    /**
     * Set the types of fields to be displayed in the drop down list.
     * <p>Example:
     * <blockquote>
     * <pre>
     * def builder = api.inputBuilderFactory().createDMFields("MyInput","DM.datamart_transaction")
     *                   .setFieldTypes(FieldType.NUMBER, FieldType.MONEY) // to select either number or money fields
     * return libs.InputLib.Input.inputParamBuilder(out, builder)
     * </pre>
     * </blockquote>
     *
     * @param types Allowed {@link FieldType}s
     * @return self
     * @see FieldType
     */
    public DMFieldInputBuilder setFieldTypes(Collection<FieldType> types) {
        writeParameterCollection(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_DM_FIELD_TYPES, types);
        return this;
    }

    private <E extends Enum<E>> void writeParameterCollection(String parameter, Collection<E> values) {
        cp.addParameterConfigEntry(parameter, values);
    }
}
