package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;

import java.util.Map;

/**
 * Builder class for input type DMFilterBuilder {@link InputType#DMFILTERBUILDER}.
 */
public class DmFilterBuilder extends AbstractInputBuilder<DmFilterBuilder, String> {

    protected DmFilterBuilder(InputBuilderContext builderContext, String paramName, String source) {
        super(builderContext, InputType.DMFILTERBUILDER, paramName);
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_DM_SOURCE_NAME, source);
    }

    /**
     * Set filters' map of the ContextParameter.
     * <p>Example:
     * <blockquote><pre>
     * Map filters = ...
     *
     * api.inputBuilderFactory()
     *         .createDmFilterBuilder("DmFilter", "source")
     *         .setFilter(filters)
     *         .buildContextParameter()
     * </pre></blockquote>
     *
     * @param filters Map of the filters
     * @return self
     */
    public DmFilterBuilder setFilter(Map filters) {
        cp.setFilter(filters);
        return this;
    }

    @Override
    protected Object _getInput() {
        return builderContext.getEngineContext().getParameter(cp.getName());
    }
}
